package work.xuye.sink.handler;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.DigestUtils;
import org.springframework.util.ObjectUtils;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.db.entity.vo.SinkParams;
import work.xuye.common.db.entity.vo.TableTemplate;
import work.xuye.common.dto.TaskVO;
import work.xuye.common.enums.ProcessMode;
import work.xuye.common.enums.ResourceStatus;
import work.xuye.common.service.UrlMD5Service;
import work.xuye.sink.service.JdbcService;
import work.xuye.sink.spel.CustomFunction;
import work.xuye.sink.spel.SpringExpressionLanguageEvaluator;

import java.util.HashMap;
import java.util.Map;

/**
 * @author xuye
 * @since 2023/3/4 00:07
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class MySqlSInk {

    private final JdbcService jdbcService;

    private final UrlMD5Service urlMD5Service;

    private final SpringExpressionLanguageEvaluator evaluator;

    private final CustomFunction customFunction;

    public Message<String> handle(Message<String> message, TaskVO taskVO) {
        String md5Digest = DigestUtils.md5DigestAsHex(message.getPayload().getBytes());

        HashMap<String, Object> resultMap = evaluator.evaluate(taskVO.getMapping().getTableMapping(), message.getPayload());
        TableTemplate tableTemplate = taskVO.getTemplate().getTableTemplate();
        ProcessMode processMode = taskVO.getTask().getProcessMode();

        if (!resultMap.containsKey(tableTemplate.getUniqueField().getFieldName())) {
            throw new RuntimeException("uniqueField [" + tableTemplate.getUniqueField().getFieldName() + "] not found");
        }
        Object uniqueValue = resultMap.get(tableTemplate.getUniqueField().getFieldName());
        String uniqueFieldValue = uniqueValue.toString();
        tableTemplate.getUniqueField().setValue(uniqueValue);
        ResourceStatus resourceStatus = urlMD5Service.getResourceStatus(uniqueFieldValue, md5Digest);
        log.info("@@@ 正在处理: " + MessageConstants.TASK_TRACE_ID + ": [{}], "
                        + MessageConstants.SOURCE_TRACE_ID + ": [{}], "
                        + MessageConstants.TRANSFORMER_TRACE_ID + ": [{}], "
                        + "{}: [{}]",
                message.getHeaders().get(MessageConstants.TASK_TRACE_ID),
                message.getHeaders().get(MessageConstants.SOURCE_TRACE_ID),
                message.getHeaders().get(MessageConstants.TRANSFORMER_TRACE_ID),
                tableTemplate.getUniqueField().getFieldName(),
                tableTemplate.getUniqueField().getValue());
        if (ResourceStatus.UNCHANGED == resourceStatus && ProcessMode.NORMAL == processMode) {
            log.info("@@ 数据未更新，无需继续处理");
            return null;
        } else {
            if (ProcessMode.DEBUG == processMode && ResourceStatus.UNCHANGED == resourceStatus) {
                log.info("@@ 数据未更新，但当前为调试模式，需继续进行处理");
            } else if (ResourceStatus.UNKNOWN == resourceStatus) {
                log.info("@@ 数据状态未知，需进行处理");
            } else if (ResourceStatus.UPDATED == resourceStatus) {
                log.info("@@ 数据相比缓存中上次记录已更新，需进行处理");
            }
        }

        for (TableTemplate.Field field : tableTemplate.getOtherFields()) {
            String fieldName = field.getFieldName();
            if (resultMap.containsKey(fieldName)) {
                Object value = resultMap.get(fieldName);
                field.setValue(value);
            }
        }

        if (!ObjectUtils.isEmpty(taskVO.getTask().getSinkParams().getTableName())) {
            tableTemplate.setTableName(taskVO.getTask().getSinkParams().getTableName());
        }

        String dataSourceName = taskVO.getTask().getSinkParams().getDataSourceName();
        Assert.notNull(dataSourceName, "dataSourceName is null");

        String selectExist = SqlGenerator.generateExistedSql(tableTemplate);
        if (jdbcService.isExist(dataSourceName, selectExist)) {
            String selectOne = SqlGenerator.generateFetchSql(tableTemplate);
            Map<String, Object> dbResult = jdbcService.getOne(dataSourceName, selectOne);

            SinkParams sinkParams = taskVO.getTask().getSinkParams();

            SinkParams.CheckUpdate checkUpdate = sinkParams.getCheckUpdate();
            String fieldName = checkUpdate.getFieldName();
            Object dbValue = dbResult.get(fieldName);
            if (ObjectUtils.isEmpty(dbValue)) {
                throw new RuntimeException("根据 [" + fieldName + "] 来判断数据是否需要更新，但是数据库不存在该字段");
            }
            Object nowValue = resultMap.get(fieldName);
            if (!ObjectUtils.isEmpty(checkUpdate.getJsonPath())) {
                dbValue = customFunction.jsonExtra(dbValue.toString(), checkUpdate.getJsonPath());
                nowValue = customFunction.jsonExtra(nowValue.toString(), checkUpdate.getJsonPath());
            }
            if (dbValue.equals(nowValue)) {
                log.info("@ 数据已存在于MySQL，且不需要更新");
                urlMD5Service.updateUrlMd5(uniqueFieldValue, md5Digest);
                return null;
            } else {
                String update = SqlGenerator.generateUpdateSql(tableTemplate);
                jdbcService.update(dataSourceName, update);
                log.info("@ 数据已存在于MySQL，已经更新数据");
            }
        } else {
            String insert = SqlGenerator.generateInsertSql(tableTemplate);
            jdbcService.update(dataSourceName, insert);
            log.info("@ 数据不存在于MySQL，已插入数据");
        }
        urlMD5Service.updateUrlMd5(uniqueFieldValue, md5Digest);
        return null;

    }

}
