package work.xuye.transformer.handler;

import com.jayway.jsonpath.JsonPath;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONArray;
import org.springframework.cloud.stream.function.StreamBridge;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.constant.StageConstants;
import work.xuye.common.db.service.TaskManager;
import work.xuye.common.dto.HttpRes;
import work.xuye.common.dto.TaskVO;
import work.xuye.common.enums.ProcessMode;
import work.xuye.common.enums.ResourceStatus;
import work.xuye.common.utils.ID;
import work.xuye.transformer.transformer.MessageTransformer;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author xuye
 * @since 2023/2/19 17:04
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class TransformApiHandler {
    private final StreamBridge streamBridge;


    private final Map<String, MessageTransformer> transformMessageMap;

    private final TaskManager taskManager;


    public List<Message<String>> handle(Message<HttpRes> message) {

        Integer taskId = (Integer) message.getHeaders().get(MessageConstants.TASK_ID);
        TaskVO taskVO = taskManager.getTaskInfoByTaskId(taskId);
        ProcessMode processMode = taskVO.getTask().getProcessMode();

        HttpRes httpRes = message.getPayload();
        log.info("@@@ 正在处理：[{}], " + MessageConstants.TASK_TRACE_ID +
                        ": [{}] " + MessageConstants.SOURCE_TRACE_ID + ": [{}]",
                httpRes.getUrl(),
                message.getHeaders().get(MessageConstants.TASK_TRACE_ID),
                message.getHeaders().get(MessageConstants.SOURCE_TRACE_ID));
        if (ResourceStatus.UNCHANGED.equals(httpRes.getResourceStatus()) && ProcessMode.NORMAL.equals(processMode)) {
            log.info("@@ 经md5判断，此URL相对上次未更新，且为正常处理模式，无需做进一步处理");
            return null;
        } else if (ProcessMode.DEBUG.equals(processMode) && ResourceStatus.UNCHANGED.equals(httpRes.getResourceStatus())) {
            log.info("@@ 经md5判断，此URL相对上次未更新，但处于调试模式，需继续进行处理");
        } else if (ResourceStatus.UNKNOWN.equals(httpRes.getResourceStatus())) {
            log.info("@@ 经md5判断，此URL为首次处理，需进行处理");
        } else if (ResourceStatus.UPDATED.equals(httpRes.getResourceStatus())) {
            log.info("@@ 经md5判断，此URL相对上次更新了，需进行处理");
        }


        //先经过xml转json等处理
        List<String> handlers = taskVO.getTask().getTransformParams().getHandlers();
        Assert.notNull(handlers, "handlers is null");

        String result = httpRes.getBody();
        for (String handler : handlers) {
            MessageTransformer messageTransformer = transformMessageMap.get(handler);
            if (messageTransformer == null) {
                log.warn("handler {} not found", handler);
                continue;
            }
            result = messageTransformer.transform(result);
        }


        ArrayList<Message<String>> results = new ArrayList<>();
        String transformerTraceId = ID.generate();
        if (!ObjectUtils.isEmpty(taskVO.getTask().getParseParams()) &&
                !ObjectUtils.isEmpty(taskVO.getTask().getParseParams().getItemsPath())) {

            Object items = JsonPath.read(result, taskVO.getTask().getParseParams().getItemsPath());
            String array = items.toString();

            JSONArray jsonArray = new JSONArray(array);
            jsonArray.forEach(item -> {
                Message<String> itemMessage = MessageBuilder.withPayload(item.toString())
                        .setHeader(MessageConstants.CURRENT_STAGE, StageConstants.TRANSFORMER)
                        .setHeader(MessageConstants.TASK_TRACE_ID, message.getHeaders().get(MessageConstants.TASK_TRACE_ID))
                        .setHeader(MessageConstants.SOURCE_TRACE_ID, message.getHeaders().get(MessageConstants.SOURCE_TRACE_ID))
                        .setHeader(MessageConstants.TRANSFORMER_TRACE_ID, transformerTraceId)
                        .setHeader(MessageConstants.TASK_ID, message.getHeaders().get(MessageConstants.TASK_ID))
                        .setHeader(MessageConstants.SEED_URL, message.getHeaders().get(MessageConstants.SEED_URL))
                        .build();
                results.add(itemMessage);
            });
            log.info("@ 下发转换后的消息，数量：{}, " + MessageConstants.TRANSFORMER_TRACE_ID + ": [{}]",
                    results.size(), transformerTraceId);
        } else {
            Message<String> itemMessage = MessageBuilder.withPayload(result)
                    .setHeader(MessageConstants.CURRENT_STAGE, StageConstants.END)
                    .setHeader(MessageConstants.TASK_TRACE_ID, message.getHeaders().get(MessageConstants.TASK_TRACE_ID))
                    .setHeader(MessageConstants.SOURCE_TRACE_ID, message.getHeaders().get(MessageConstants.SOURCE_TRACE_ID))
                    .setHeader(MessageConstants.TRANSFORMER_TRACE_ID, transformerTraceId)
                    .setHeader(MessageConstants.TASK_ID, message.getHeaders().get(MessageConstants.TASK_ID))
                    .setHeader(MessageConstants.SEED_URL, message.getHeaders().get(MessageConstants.SEED_URL))
                    .build();
            results.add(itemMessage);
            log.info("@ 该消息未进行转换，直接下发，后续无需处理，" + MessageConstants.TRANSFORMER_TRACE_ID + ": [{}]",
                    transformerTraceId);
        }
        return results;


    }
}
